ifndef fxa_wdl;
define fxa_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 5.0
// engineReq: 6.4
// date: 061011
//
// title: fxa (01)
// class: ENVIR
// type: USER
// image: fxa01.pcx
// help: Effect library
//
// needs: gid01.wdl, trigger00.wdl, particle00.wdl
//
// external:
//
// prefix: fxa_
// idcode: 02E
//
// Dev Note: some of these scripts are modified version of code writen
//  by Dan Silverman & Czeslaw Gorski
//
// ENDHEADER
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------
// Smoke

// entry: First Smog Puff bitmap filename
// id: 1
string fxa_smog1_str = <smog_01.tga>;

// entry: Second Smog Puff bitmap filename
// id: 2
string fxa_smog2_str = <smog_02.tga>;

// entry: Third Smog Puff bitmap filename
// id: 3
string fxa_smog3_str = <smog_03.tga>;


// Desc: Update smoke puff
function FXA_Smog_Update()
{
	while(my.alpha > 1)
	{
		my.alpha -= my.skill8 * time_step;

		my.roll += my.skill7*time_step;

		my.scale_x += my.skill5*time_step;
		my.scale_y = my.scale_x;

		my.z += (random(1) + my.skill6)*time_step;       		// up drift
		my.skill4 -= my.skill4*(max(0.01,(my.skill10*time_step)));	// calc drag
		my.y += (my.skill4 * sin(my.skill40))*time_step;      // update xy pos with vel
		my.x += (my.skill4 * cos(my.skill40))*time_step;
		wait(1);
	}

	if(you != NULL)
	{
		// reduce the counter on the puff emitter
		you.skill44 -= 1;
	}
	ent_remove(me);
}


// Desc: Init smoke puff
function FXA_Smog_Init()
{
	my.red = 100;
	my.green = 70;
	my.blue = 70;

	// starting values
	my.roll = random(360);
	my.scale_x = you.skill9;
	my.scale_y = my.scale_x;

	my.skill2 += 0.03;
	my.skill4 = you.skill4;
	my.skill5 = you.skill5;
	my.skill6 = you.skill6;
	my.skill7 = you.skill7;
	my.skill8 = you.skill8;
	my.skill10 = you.skill10;
	my.skill40 = you.pan;

	my.facing = on;
	my.alpha = 80;
	my.ambient = 100;
	my.transparent = 1;
	my.passable = ON;
	my.bright =1;
	my.flare =1 ;
	FXA_Smog_Update();
}

// action: FXA_Smog
// title: Smoke Machine
// Desc: Emits 'Puffs' of smoke.
// help: Smoke moves up and out (using this entity's pan value)
//
//
// section: Trigger
//
// skill12: ID 0
// help: Activate when trigger is set? (0 if always active)
//
// section: Smoke
//
// skill1: Bitmap	1
// help: Select bitmap (choose bitmap in script).
// cntl: spin 1 3 1
//
// skill2: PuffDelay	4
// help: Time between puffs of smoke (in ticks)
// cntl: spin 1 x 1
//
// skill3: MaxPuffs 16
// help: Max number of puffs that can be in use
// cntl: spin 1 100 1
//
//	skill9: StartScale 0.1
// help: Starting scale of the puff
// cntl: spin 0 x 0.1
//
// skill5: Scale 0.05
// help: How quickly the puff scales up over time
//
// skill4: Speed 8
// help: Speed that the puff moves away from emitter
//
// skill10: Drag 0.08
// help: Drag slows puff vel over time
// cntl: spin 0 1 0.01
//
// skill6: UpDrift 1.5
// help: Speed in the z-vec (up or down)
//
// skill7: Rotation 4
// help: Rotation speed
//
// skill8: Fade 1.5
// help: How quickly the puff fades.
// cntl: spin 0.1 x 0.1
action FXA_Smog
{
	my.invisible = on;
	my.passable = on;
	if(my.skill2 == 0) {my.skill2 = 2; }
	if(my.skill3 == 0) {my.skill3 = 10;}
	if(my.skill4 == 0) {my.skill4 = 2;}
	if(my.skill5 == 0) {my.skill5 = 0.1;}
	if(my.skill6 == 0) {my.skill6 = 0.1;}
	if(my.skill7 == 0) {my.skill7 = 1.5;}
	if(my.skill8 == 0) {my.skill8 = 1.8;}


	my.skill2 = max(0.01,(my.skill2/16));

	my.skill44 = 0;		// current number of smoke puffs

	while(1)
	{

		// if max puffs is greater than the current number
		if(my.skill44 < my.skill3)
		{
	 		if((my.skill12 == 0) || Trigger00_Check(my.skill12,trigger00_bitlist) == 1)
 			{
				if(my.skill1 < 2)
				{
					ent_create(fxa_smog1_str,my.x,FXA_Smog_Init);
				}
				else
				{
					if(my.skill1 < 3)
					{
						ent_create(fxa_smog2_str,my.x,FXA_Smog_Init);
					}
					else
					{
   					ent_create(fxa_smog3_str,my.x,FXA_Smog_Init);
  					}
  				}
	  			my.skill44 += 1;
			}
		}
		sleep(my.skill2);
		wait(1);		// at low framerates, sleep may be ignored is skill2 < 1/2 frametime
	}
}


// ---------------------------------------------------------------------
// Explosion


define	fxa_exp_maxvel = 3;
bmap	fxa_flamesmk1_bmp = <smkpart4.tga>;
bmap	fxa_flamesmk2_bmp = <smkpart5.tga>;


string fxa_placehold_str = <arrow_blue.pcx>;

// Particle updater, fade alpha and color to redish/black while expanding size.
function	FXA_Exp1_Update1()
{
	my.alpha -= 1*time_step;
	if(my.alpha < 0) { my.lifespan = 0; }

  	my.red -= 9*time_step;		my.red = max(my.red,8);
	my.green -= 16*time_step;	my.green = max(my.green,0);
	my.blue -= 32*time_step;	my.blue = max(my.blue,0);

	if (my.size<128)
	{
		my.size += 3*time_step;
	}
}


// Particle init, starts redish/white and fades to black (flame2smoke).
function FXA_Exp1_FX()
{
	// starting color
	my.red   = 180 + random(75);
	my.green = 95 + random(100);
	my.blue  = 80 + random(100);

	my.lifespan = 1600; // don't time out, fade away

	my.move = ON;

	my.transparent = ON;
	my.bright = OFF;
	my.light = ON;
	my.alpha = 15 + random(25);

	my.size = 4 + random(8);

	if(random(5) < 3)
	{
 		my.bmap = fxa_flamesmk2_bmp;
	}
	else
	{
 		my.bmap = fxa_flamesmk1_bmp;
	}

	my.function = FXA_Exp1_Update1;
}

// Entity moves outword in a random direction from its starting point,
//    leaving a fire2Smoke trail of particles.
function FXA_Exp1_Trail()
{
 	my.invisible = ON;

	// SKILL1-3: Starting speed vertor
	my.skill1 = random(fxa_exp_maxvel) - (fxa_exp_maxvel/2);
	my.skill2 = random(fxa_exp_maxvel) - (fxa_exp_maxvel/2);
	my.skill3 = random(fxa_exp_maxvel);

	// SKILL7: Total lifespan (in ticks) + 10
	my.skill7 = 8 + random(16);
	// SKILL8: Current lifespan left
	my.skill8 = my.skill7;
	while(my.skill8 > 10) 	// 10 because we want to stop making flames while still moving
	{
		my.skill8 -= time_step;

		// SKILL9: Square of (Ratio of remaining life over total life)
		my.skill9 = (my.skill8/my.skill7);          // time left/total time  (1->0)
		my.skill9 *= my.skill9;							  // skill9 = (1->0)^2

		// drop first particle, do half move
		if(random(4*my.skill9) > 1)  // creates less smoke as entity slows
		{
			// drop flame2smoke
			temp.x = random(fxa_exp_maxvel) - (fxa_exp_maxvel/2);
			temp.y = random(fxa_exp_maxvel) - (fxa_exp_maxvel/2);
			temp.z = random(fxa_exp_maxvel);
			effect(FXA_Exp1_FX,1,my.x,temp);
		}
		vec_set(my.skill4,my.skill1);          // base cur speed of init speed
		vec_scale(my.skill4,(my.skill9*6*time_step));
		vec_add(my.x,my.skill4);

		// drop second particle, finish move
		if(random(4*my.skill9) > 1)  // creates less smoke as entity slows
		{
			// drop flame2smoke
			temp.x = random(fxa_exp_maxvel) - (fxa_exp_maxvel/2);
			temp.y = random(fxa_exp_maxvel) - (fxa_exp_maxvel/2);
			temp.z = random(fxa_exp_maxvel);
			effect(FXA_Exp1_FX,1,my.x,temp);
		}
		// move calculated above
		vec_add(my.x,my.skill4);

		wait(1);
	}

	ent_remove(me);
}

// Rapidly expand a sprite to simulate the "blast ring" of an explosion
function	FXA_Exp1_Ring()
{
	my.passable = ON;

	my.facing = OFF;
	my.oriented = ON;
	my.pan = 0;
	my.tilt = 90;
	my.roll = 0;

	my.transparent = ON;
	my.alpha = 15+random(25);
	my.bright = OFF;
	my.light = ON;

	while(my.alpha > 0)
	{
		my.alpha -= 2*time_step;   // at starting alpha=32; flash lasts ~1 second
		my.scale_x += my.scale_x*0.25*time_step;
		my.scale_y += my.scale_x*0.25*time_step;
		wait(1);
	}

	ent_remove(me);

}

// Rapidly expand a sprite to simulate the flash of an explosion
function	FXA_Exp1_Flash()
{
	my.passable = ON;

	// facing it towards the camera gives it a 3d feel
	my.facing = ON;
	my.oriented = OFF;

	my.transparent = ON;
	my.alpha = 45+random(10);
	my.bright = OFF;
	my.light = ON;

	// fade quickly over time
	while(my.alpha > 0)
	{
		my.alpha -= 12*time_step;	// at starting alpha=50; flash lasts ~0.26 seconds
		my.scale_x += my.scale_x*0.75*time_step;
		my.scale_y += my.scale_x*0.75*time_step;
		wait(1);
	}

	ent_remove(me);
}


/*
// This is one example on how to create an explosion effect.
//
action FXA_ExplosionTest
{

	while(1)
	{
		// cycle every 3 seconds
		sleep(3);

		// flash
		ent_createlocal("flash3.tga",my.x,FXA_Exp1_Flash);

		// expanding ring
		ent_createlocal("flash1.tga",my.x,FXA_Exp1_Ring);

		// fireball
		ent_createlocal(fxa_placehold_str,my.x,FXA_Exp1_Trail);
		ent_createlocal(fxa_placehold_str,my.x,FXA_Exp1_Trail);
		ent_createlocal(fxa_placehold_str,my.x,FXA_Exp1_Trail);
		ent_createlocal(fxa_placehold_str,my.x,FXA_Exp1_Trail);
		ent_createlocal(fxa_placehold_str,my.x,FXA_Exp1_Trail);
		ent_createlocal(fxa_placehold_str,my.x,FXA_Exp1_Trail);
		ent_createlocal(fxa_placehold_str,my.x,FXA_Exp1_Trail);
		ent_createlocal(fxa_placehold_str,my.x,FXA_Exp1_Trail);

	}
}
*/


// ---------------------------------------------------------------------
// Short Circuit


// entry: First sound file used in short circuit
// id: 4
sound fxa_circuit1_wav = <circuit1.wav>;

// entry: Second sound file used in short circuit
// id: 5
sound fxa_circuit2_wav = <circuit2.wav>;

var_nsave  fxa_circuit_handle;

function FXA_ShortCircuit_Update()
{
	my.skill_X+=1*time_step;
	if(my.skill_x > 0.145 ) {  my.gravity= 3.462 ;  }
	my.skill_b += 0.03;
	my.skill_a = my.skill_c  - my.skill_b;
	if(my.skill_a < 0) { my.lifespan = 0; }
	if(my.alpha > 10) {my.alpha -=1 * time_step;}
}

// Desc: Init particle function for shortcircuit
function FXA_ShortCircuit_Part()
{
	my.red = you.skill6;
	my.green = you.skill7;
	my.blue = you.skill8;
	my.skill_c = you.skill9;
	if(my.skill_a < 0) { my.skill_a = 0; my.skill_b = 0; }
	my.vel_x = random( 15.500 ) - (15.500 /2);
	my.vel_y = random( 3 ) - (3 /2);
	my.vel_z = random( 3 );
	if(you.skill11 < 1){my.size =1 ;} else {my.size = you.skill11;}
	my.move =1;
	my.alpha = 50;
	my.transparent = 1;
	my.streak =0;
	my.bright =1;
	my.flare =1;
	my.function = FXA_ShortCircuit_Update;
}

// action: FXA_ShortCircuit
// title: Short Circuit Sparker
// help:
// Desc: Emits shower of sparks with making "zap" noises.
//
//
// section: Trigger
//
// help: Activate when trigger is set? (0 if always active)
// skill12: ID 0
//
// section: Dynamic Light
//
// skill1: Red 133
// cntl: spin 0 255 1
//
// skill2: Green 194
// cntl: spin 0 255 1
//
// skill3: Blue 201
// cntl: spin 0 255 1
//
// cntl: 0 x 1
// skill5: LightRange 250
//
// section: Spark
//
// skill11: Size 1
// cntl: 1 x 1
//
// skill6: Red 255
// cntl: spin 0 255 1
//
// skill7: Green 255
// cntl: spin 0 255 1
//
// skill8: Blue 203
// cntl: spin 0 255 1
//
// skill9: Lifespan 250
// cntl: 1 x 1
//
// skill14: PerSpark  55
// help: Percent chance that this emitter will spark every 1/4 second.
// cntl: 1 100 1
//
// skill15: DurSpark  6
// help: Duration of each spark shower (in ticks).
// cntl: 1 x 1
action FXA_ShortCircuit
{
	my.invisible = on;
	my.passable = on;

	// activate dynamic light color
	my.red = my.skill1;
	my.green = my.skill2;
	my.blue = my.skill3;

	my.skill15 /= 2;	// random duration time = base + random(base)


	while(1)
	{
	 	if((my.skill12 == 0) || Trigger00_Check(my.skill12,trigger00_bitlist) == 1)
		{
			if(random(100) < my.skill14) // percent chance of activating everything 1/4 second
			{
				my.skill21 = my.skill15+random(my.skill15);   // duration

				while(my.skill21 > 0)
				{
					my.lightrange = my.skill5 * (random(0.8)+0.2);  // 20 to 100% the max lightrange

					effect(FXA_ShortCircuit_Part,(20*time_step),my.x,nullvector);

					// play a random "zap" sound if no sound is playing
					if(!snd_playing(fxa_circuit_handle))
					{
						if(random(2)> 1) {fxa_circuit_handle = ent_playsound(my,fxa_circuit1_wav,100);}
						else {fxa_circuit_handle = ent_playsound(my,fxa_circuit2_wav,100);}
					}

					my.skill21 -= time_step;
					wait(1);
				}
			}
		}
		my.lightrange = 1;
		sleep(0.25);
	}
}




// ---------------------------------------------------------------------
// Lights

// action: FXA_LightBlink
// title: Blinking Light
// Desc: Cycle a dynamic light with 4 ranges and 4 delays.
// Desc: Can use trigger to only active light when trigger is set.
// Desc: (Negitive trigger, light is active when trigger is not set).
//
// ------------------------------------------
// skill12: Trigger 0
// help: If 0, no trigger is needed (always on)
// help: Otherwise, the light only blinks when the matching trigger is set.
// help: If the value is negitive, the light only blicks when the matching trigger is NOT set.
//
// ------------------------------------------
// section: Light Color
// skill1: Red 200
// cntl: spin 0 255 1
//
// skill2: Green 200
// cntl: spin 0 255 1
//
// skill3: Blue 200
// cntl: spin 0 255 1
//
// ------------------------------------------
// section: First Range
// skill4: Range1 200
// help: Range in quants
//
// skill5: Delay1 3
// help: Delay in seconds
// ------------------------------------------
// section: Second Range
//	skill6: Range2 175
// help: Range in quants
//
// skill7: Delay2 1
// help: Delay in seconds
// ------------------------------------------
// section: Third Range
//	skill8: Range3 300
// help: Range in quants
//
// skill9: Delay3 1.25
// help: Delay in seconds
// ------------------------------------------
// section: Fourth Range
//	skill10: Range4 175
// help: Range in quants
//
// skill11: Delay4 1
// help: Delay in seconds
//
// flag1: Randomize? 1
// help: randomize time?
action FXA_LightBlink
{
	my.red = my.skill1;
	my.green = my.skill2;
	my.blue = my.skill3;

	if(my.skill12 < 0)
	{
		// negitive trigger action
		my.skill12 = abs(my.skill12);
		my.skill13 = 0;	// on if trigger is not set!
	}
	else
	{
 		my.skill13 = 1;	// on if trigger is set!
	}
 	// cap trigger value to max
	if(my.skill12 > (trigger00_max_value_const*trigger00_bitlist_size_const))
	{
		my.skill12 = (trigger00_max_value_const*trigger00_bitlist_size_const);
	}

	while(1)
	{
		if((my.skill12 == 0)
			|| (Trigger00_Check(my.skill12,trigger00_bitlist) == my.skill13))   // trigger active/disactive?
		{
				// note: must finish cycle even if trigger is turned off!

				if(my.skill5 > 0)
				{
					my.lightrange = my.skill4; // lightrange 1
					if(my.flag1 == on) { sleep(random (my.skill5)-(my.skill5/2)); }
					else { sleep(my.skill5); }
				}
				if(my.skill7 > 0)
				{
					my.lightrange = my.skill6; // lightrange 2
					if(my.flag1 == on) { sleep(random (my.skill7)-(my.skill7/2)); }
					else { sleep(my.skill7); }
				}
				if(my.skill9 > 0)
				{
					my.lightrange = my.skill8; // lightrange 3
					if(my.flag1 == on) { sleep(random (my.skill9)-(my.skill9/2)); }
					else { sleep(my.skill9); }
				}
				if(my.skill11 > 0)
				{
					my.lightrange = my.skill10;// lightrange 4
					if(my.flag1 == on) { sleep(random (my.skill11)-(my.skill11/2)); }
					else { sleep(my.skill11); }
				}
			}
			else
			{
				// light off
				my.lightrange = 0;
				sleep(0.5);
			}
		}
	}
}

// ---------------------------------------------------------------------
// Misc


// action: FXA_Transparent
// title: Transparent Entity
// Desc: Make an entity transparent
//
// skill1: Alpha 25
//	help: The alpha value for this entity (0-100)
// cntl: spin 0 100 1
action FXA_Transparent
{
	MY.TRANSPARENT = ON;
	my.alpha = my.skill1;
}


// ---------------------------------------------------------------------
// View effects
VIEW fxa_mirror_view { };

// action: FXA_Mirror
// title: Mirror
// Desc: Create a floor mirror.
// Code to simulate a floor mirror by generating
// a vertically flipped camera view from below through the floor
action FXA_Mirror
{
// initialize camera portal for realtime reflections
ifdef CAPS_MIRROR;
	camera.portal = fxa_mirror_view;
	fxa_mirror_view.noshadow = on;	// suppress shadows in the mirror
	fxa_mirror_view.portalclip = on;	// clip at portal plane
	while(1)
	{
		proc_late();	// place it at the end of the function list - the camera must be moved before
		fxa_mirror_view.genius = camera.genius;
		fxa_mirror_view.aspect = -camera.aspect;	// flip the image upside down
		fxa_mirror_view.arc 	  = camera.arc;
		fxa_mirror_view.fog 	  = camera.fog;
		fxa_mirror_view.fog_start = camera.fog_start;
		fxa_mirror_view.fog_end   = camera.fog_end;
		fxa_mirror_view.x 	= camera.x;
		fxa_mirror_view.y 	= camera.y;
		fxa_mirror_view.z 	= 2*camera.portal_z-camera.z;	// move the camera at its mirror position
		fxa_mirror_view.pan  = camera.pan;
		fxa_mirror_view.tilt = -camera.tilt;	// flip the vertical camera angle
		fxa_mirror_view.roll = -camera.roll;
		wait(1);
	}
endif;
	return;
}





endif;	// fxa01_wdl