ifndef plBipedInput_wdl;
define plBipedInput_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 5.0
// engineReq: 6.4 PRO
// date: 061011
//
// title: Biped Player Input (01)
// class: PLAYER
// type: SUPPORT
// image: plBipedInput01.pcx
// help: Set up and poll user input while the player is using the biped model.
//
//
// needs: miscInput01.wdl
//
// uses: plSelect::plSelect_curr_model,
//
// prefix: plBipedInput01_
// idcode: 014
//
// DevNote: plBipedIn01 --> plBipedInput01
//
// ENDHEADER
// ---------------------------------------------------------------------


// entry: Enable "mouse look"?
// help: If 1 the headangle/rotation of the player is effected by the mouse.
// cntl: spin 0 1 1
// id: 1
var	plBipedInput01_mouse_look_q = 1;

// entry: invert "mouse look"?
// help: If 1 we invert the effect the mouse has on headangle tilt.
// cntl: spin 0 1 1
// id: 2
var	plBipedInput01_mouse_look_invt_q = 1;

// entry: Mouse sensitivity
// help: The larger the value the more sensitive the view is to mouse look
// id: 3
var	plBipedInput01_mouse_look_factor = 5.5;

// section: Keys

// entry: Move forward key
// id: 4
string plBipedInput01_move_for_key_c = "W";
var	plBipedInput01_move_for_scancode = 0;
// entry: Move backwards key
// id: 5
string plBipedInput01_move_back_key_c = "S";
var	plBipedInput01_move_back_scancode = 0;
// entry: Step left key
// id: 6
string plBipedInput01_move_left_key_c = "A";
var	plBipedInput01_move_lt_scancode = 0;
// entry: Step right key
// id: 7
string plBipedInput01_move_right_key_c = "D";
var	plBipedInput01_move_rt_scancode = 0;

// entry: Rotate CW key
// id: 8
string plBipedInput01_rotateCW_key_c = "";
var	plBipedInput01_rotateCW_scancode = 0;
// entry: Rotate CCW key
// id: 9
string plBipedInput01_rotateCCW_key_c = "";
var	plBipedInput01_rotateCCW_scancode = 0;

// entry: Jump/Climb key
// id: 10
string plBipedInput01_jumpup_key_c = "space";
var	plBipedInput01_jumpup_scancode = 0;
// entry: Duck/Down key
// id: 11
string plBipedInput01_duckdown_key_c = "C";
var	plBipedInput01_duckdown_scancode = 0;

// entry: Lookup key
// id: 12
string plBipedInput01_lookup_key_c = "pgup";
var	plBipedInput01_lookup_scancode = 0;
// entry: Lookdown key
// id: 13
string plBipedInput01_lookdown_key_c = "pgdn";
var	plBipedInput01_lookdown_scancode = 0;


// entry: walk/slow key
// help: slows the player's movement
// id: 14
string plBipedInput01_slow_key_c = "shiftl";
var	plBipedInput01_slow_scancode = 0;

// entry: Walk/slow Value
// help: multiplication factor used to reduce/increase the player's
// help:speed when the walk/slow key is pressed.
// id: 15
define	plBiped01_force_slow_const = 0.5;


// entry: Preform "Activation Scan"
// help: Used to open doors, pick up items, etc.
// id: 16
string	plBipedInput01_activate_key_c = "ctrl";
var plBipedInput01_activate_scancode = 0;



// section: Misc.
var	plBipedInput01_key_delay;

// entry: Key Delay (ticks)
// help: Time (ticks) between delayed keypresses
// help: Used for keys like "activation scan"
// id: 17
define plBipedInput01_key_delay_const = 16;



//-----------------------------------------
// Section: Joystick Input

// entry_drop: Joystick
// help: 0: No joystick input
// help: 1: Treat joystick input as "digital" (on or off)
// help: 2: Treat joystick input as "analog"
// help: 3: XBox360 controller
// cntl: drop_list 0 1 2 3
// id: 18
var	plBipedInput01_qJoystick = 0;

// entry: Joystick Activation Scan button
// id: 20
string plBipedInput01_activate_but_c = "joy_1";
var	plBipedInput01_activate_but_scancode = 0;

// entry: Joystick Jump button
// id: 19
string plBipedInput01_jumpup_but_c = "joy_2";
var	plBipedInput01_jumpup_but_scancode = 0;


// entry: Use Item 1 button
// id: 29
string plBipedInput01_use1_but_c = "joy_3";
var plBipedInput01_use1_but_scancode = 0;

// entry: Use Item 2 button
// id: 30
string plBipedInput01_use2_but_c = "joy_4";
var plBipedInput01_use2_but_scancode = 0;

// entry: Joystick Duck Scan button
// id: 25
string plBipedInput01_duckdown_but_c = "joy_9";
var	plBipedInput01_duckdown_but_scancode = 0;

// entry: Joystick Prev Item button
// id: 26
string plBipedInput01_previtem_but_c = "joy_5";
var	plBipedInput01_previtem_but_scancode = 0;

// entry: Joystick Next Item button
// id: 27
string plBipedInput01_nextitem_but_c = "joy_6";
var	plBipedInput01_nextitem_but_scancode = 0;


// entry: Joy "Dead" X-axis
// help: "Dead Zone" for the joystick in the X-axis
// id: 21
define	plBipedInput01_joyNullX = 50;

// entry: Joy "Dead" Y-axis
// help: "Dead Zone" for the joystick in the Y-axis
// id: 22
define	plBipedInput01_joyNullY = 50;

// entry: "Joy-look" tilt const
// help: Higher numbers equal faster looking up/down
// id: 31
define	plBipedInput01_joyTiltConst = 50;

// entry: "Joy-look" pan const
// help: Higher numbers equal faster turning left/right
// id: 32
define	plBipedInput01_joyPanConst = 50;

// entry: XBox, Right Trigger Point
// help: If the right finger trigger is greater than this value, item 1 is used
// help: (Only works if joystick is set to XBox360 Controller
// id: 23
define   plBipedInput01_joyRawZ_PosTrigger = 120;

// entry: XBox, Left Trigger Point
// help: If the right finger trigger is greater than this value. item 2 is used
// help: (Only works if joystick is set to XBox360 Controller
// id: 24
define   plBipedInput01_joyRawZ_NegTrigger = 120;





// ---------------------------------------------------------------------
// External Variables

// From plBiped01.wdl
var	plBiped01_force_vec[3];   	// x,y,z forces
var	plBiped01_ang_force_vec[3];	// pan,tile,roll forces
var	plBiped01_flag_array[3];			// 48 flags

define plBipedInput01_flag0_activate = 1;
define plBipedInput01_flag0_use_1 = 2;
define plBipedInput01_flag0_use_2 = 4;
define plBipedInput01_flag0_prev = 8;
define plBipedInput01_flag0_next = 16;

// ---------------------------------------------------------------------
// Exported Functions
function PlBipedInput01_Update();


// ---------------------------------------------------------------------
// enable: print "debug output" for player on screen?
// id: 28
//define PLBIPEDINPUT_DEBUG;

ifdef PLBIPEDINPUT_DEBUG;
font plBipedInput01_debug_font = <ackfont.pcx>,6,9;

var  plBipedInput01_debug_data[15];
panel plBipedInput01_debug_pan
{
	pos_x = 170;
	pos_y = 10;
	digits = 0,  0, 5, plBipedInput01_debug_font, 1, num_joysticks;  //

	digits = 0, 14, 5, plBipedInput01_debug_font, 1, joy_raw.x;  //
	digits = 0, 23, 5, plBipedInput01_debug_font, 1, joy_raw.y;	//
	digits = 0, 32, 5, plBipedInput01_debug_font, 1, joy_raw.z; //

	digits = 0, 50, 5, plBipedInput01_debug_font, 1, joy_rot.x; //
	digits = 0, 59, 5, plBipedInput01_debug_font, 1, joy_rot.y; //
  	digits = 0, 68, 5, plBipedInput01_debug_font, 1, joy_rot.z; //

   digits = 0, 86,  5, plBipedInput01_debug_font, 1, joy_1;
   digits = 0, 86, 15, plBipedInput01_debug_font, 1, joy_2;
   digits = 0, 86, 25, plBipedInput01_debug_font, 1, joy_3;
   digits = 0, 86, 35, plBipedInput01_debug_font, 1, joy_4;
	digits = 0, 95,  5, plBipedInput01_debug_font, 1, joy_5; //
	digits = 0, 95, 15, plBipedInput01_debug_font, 1, joy_6; //
	digits = 0, 95, 25, plBipedInput01_debug_font, 1, joy_7; //
	digits = 0, 95, 35, plBipedInput01_debug_font, 1, joy_8; //
	digits = 0,104,  5, plBipedInput01_debug_font, 1, joy_9; //
	digits = 0,104, 15, plBipedInput01_debug_font, 1, joy_10; //
	digits = 0,104, 25, plBipedInput01_debug_font, 1, joy_11; //
	digits = 0,104, 35, plBipedInput01_debug_font, 1, joy_12; //

	flags = refresh, d3d, visible;

}

endif;


// ---------------------------------------------------------------------
// Local Functions

// Desc: takes the current values stored in the user defined keys and creates scancodes
function	plBipedInput01_SetKeys()
{
	// use MiscInput01_Key_For_Str() to convert strings into scancodes

	// forward key
	str_cpy(miscInput01_temp_str,plBipedInput01_move_for_key_c);
	plBipedInput01_move_for_scancode = MiscInput01_Key_For_Str();
	// backwards key
	str_cpy(miscInput01_temp_str,plBipedInput01_move_back_key_c);
	plBipedInput01_move_back_scancode = MiscInput01_Key_For_Str();

	// left key
	str_cpy(miscInput01_temp_str,plBipedInput01_move_left_key_c);
	plBipedInput01_move_lt_scancode = MiscInput01_Key_For_Str();
	// right key
	str_cpy(miscInput01_temp_str,plBipedInput01_move_right_key_c);
	plBipedInput01_move_rt_scancode = MiscInput01_Key_For_Str();

	// rotate clockwise key
	str_cpy(miscInput01_temp_str,plBipedInput01_rotateCW_key_c);
	plBipedInput01_rotateCW_scancode = MiscInput01_Key_For_Str();
	// rotate ccw key
	str_cpy(miscInput01_temp_str,plBipedInput01_rotateCCW_key_c);
	plBipedInput01_rotateCCW_scancode = MiscInput01_Key_For_Str();


	// jump/up key
	str_cpy(miscInput01_temp_str,plBipedInput01_jumpup_key_c);
	plBipedInput01_jumpup_scancode = MiscInput01_Key_For_Str();
	// activation button
	str_cpy(miscInput01_temp_str,plBipedInput01_jumpup_but_c);
	plBipedInput01_jumpup_but_scancode = MiscInput01_Key_For_Str();

	// duck/down key
	str_cpy(miscInput01_temp_str,plBipedInput01_duckdown_key_c);
	plBipedInput01_duckdown_scancode = MiscInput01_Key_For_Str();
	str_cpy(miscInput01_temp_str,plBipedInput01_duckdown_but_c);
	plBipedInput01_duckdown_but_scancode = MiscInput01_Key_For_Str();



	// look up key
	str_cpy(miscInput01_temp_str,plBipedInput01_lookup_key_c);
	plBipedInput01_lookup_scancode = MiscInput01_Key_For_Str();
	// look down key
	str_cpy(miscInput01_temp_str,plBipedInput01_lookdown_key_c);
	plBipedInput01_lookdown_scancode = MiscInput01_Key_For_Str();

	// slow/walk key
	str_cpy(miscInput01_temp_str,plBipedInput01_slow_key_c);
	plBipedInput01_slow_scancode = MiscInput01_Key_For_Str();

	// activation key
	str_cpy(miscInput01_temp_str,plBipedInput01_activate_key_c);
	plBipedInput01_activate_scancode = MiscInput01_Key_For_Str();

	// activation button
	str_cpy(miscInput01_temp_str,plBipedInput01_activate_but_c);
	plBipedInput01_activate_but_scancode = MiscInput01_Key_For_Str();

	// next item button
	str_cpy(miscInput01_temp_str,plBipedInput01_nextitem_but_c);
	plBipedInput01_nextitem_but_scancode = MiscInput01_Key_For_Str();

	// prev item button
	str_cpy(miscInput01_temp_str,plBipedInput01_previtem_but_c);
	plBipedInput01_previtem_but_scancode = MiscInput01_Key_For_Str();

	// use item 1 button
	str_cpy(miscInput01_temp_str,plBipedInput01_use1_but_c);
	plBipedInput01_use1_but_scancode = MiscInput01_Key_For_Str();

	// use item 2 button
	str_cpy(miscInput01_temp_str,plBipedInput01_use2_but_c);
	plBipedInput01_use2_but_scancode = MiscInput01_Key_For_Str();
}


// ---------------------------------------------------------------------
// Starter Functions

// Desc: Initialize the starting values
starter PlBipedInput01_Init()
{
	diag("\nWDL-Loaded:plBipedInput01.wdl");


	// Init key commands
	plBipedInput01_SetKeys();

}




// ---------------------------------------------------------------------
// External Functions

// Desc: Update values according to player input
// Note: called by plBiped01 once a frame while plBiped model is active
//
// Modifies: plBiped01_force.x/y/z
//				 plBiped01_ang_force.pan/tilt/roll
//				 plBiped01_flag_array	([0] 0-15, [1] 16-31, [2] 32-47)
function PlBipedInput01_Update()
{
	// zero out forces and flags
	vec_set(plBiped01_force_vec,nullvector);
	vec_set(plBiped01_ang_force_vec,nullvector);
	vec_set(plBiped01_flag_array,nullvector); 	// NOTE: change if size of plBiped_flag_array != 3

	if(ON == enable_key)
	{
		// check for keyboard/mouse_button/joy_button input
		if(Key_Pressed(plBipedInput01_move_for_scancode))
		{
  			plBiped01_force_vec.x += 1;	// forward force
		}
  		if(Key_Pressed(plBipedInput01_move_back_scancode))
		{
  			plBiped01_force_vec.x -= 1;	// backwards force
		}
		if(Key_Pressed(plBipedInput01_move_lt_scancode))
		{
  			plBiped01_force_vec.y += 1;	// step left force
		}
		if(Key_Pressed(plBipedInput01_move_rt_scancode))
		{
  			plBiped01_force_vec.y -= 1;	// step right force
		}
		if(Key_Pressed(plBipedInput01_jumpup_scancode)
			|| Key_Pressed(plBipedInput01_jumpup_but_scancode))
		{
  			plBiped01_force_vec.z += 1;	// jump up force
		}
		if(Key_Pressed(plBipedInput01_duckdown_scancode)
			|| Key_Pressed(plBipedInput01_duckdown_but_scancode))
		{
  			plBiped01_force_vec.z -= 1;	// duck down force
		}
		if(Key_Pressed(plBipedInput01_rotateCW_scancode))
		{
			plBiped01_ang_force_vec.pan -= 1;    // rotate clockwise force
		}
		if(Key_Pressed(plBipedInput01_rotateCCW_scancode))
		{
			plBiped01_ang_force_vec.pan += 1;    // rotate clockwise force
		}

		if(Key_Pressed(plBipedInput01_lookup_scancode))
		{
			plBiped01_ang_force_vec.tilt += 1;    // look up
		}
		if(Key_Pressed(plBipedInput01_lookdown_scancode))
		{
			plBiped01_ang_force_vec.tilt -= 1;    // look down
		}


		if(Key_Pressed(plBipedInput01_use1_but_scancode))
		{
 			plBiped01_flag_array[0] |= plBipedInput01_flag0_use_1;	// set flag to use item 1 (i.e. gun)
		}
		if(Key_Pressed(plBipedInput01_use2_but_scancode))
		{
 			plBiped01_flag_array[0] |= plBipedInput01_flag0_use_2;	// set flag to use item 2
 		}


		if(Key_Pressed(plBipedInput01_activate_scancode)
			|| Key_Pressed(plBipedInput01_activate_but_scancode))
		{
			plBiped01_flag_array[0] |= plBipedInput01_flag0_activate;	// set flag to preform activation scan
		}

		//--------------------------------------------------------------
		// Joystick/pad
		// NOTE: These override the keyboard movement keys
		if(plBipedInput01_qJoystick > 0)
		{
			if(joy_raw.y<-plBipedInput01_joyNullY || joy_raw.y>plBipedInput01_joyNullY)
			{
				// treat gamepad as absolute device which is more
				// like a keyboard than a joystick
				if(plBipedInput01_qJoystick == 1)
				{
					 plBiped01_force_vec.x = -sign(joy_raw.y);
				}
				else
				{
					// using analog joystick
					plBiped01_force_vec.x = -(joy_raw.y-plBipedInput01_joyNullY) / (255-plBipedInput01_joyNullY);
				}
			}

			if(joy_raw.x < -plBipedInput01_joyNullX || joy_raw.x > plBipedInput01_joyNullX)
			{
				// treat gamepad as absolute device which is more
				// like a keyboard than a joystick
				if(plBipedInput01_qJoystick == 1)
				{
					 plBiped01_force_vec.y = -sign(joy_raw.x);
				}
				else
				{
					// using analog joystick
					plBiped01_force_vec.y = -(joy_raw.x-plBipedInput01_joyNullX) / (255-plBipedInput01_joyNullX);
				}
			}

			// XBox360 style controller  (3)
			if(plBipedInput01_qJoystick > 2)
			{


				// right analong controlls pan and tilt
 				if(joy_rot.y < -plBipedInput01_joyNullY || joy_rot.y > plBipedInput01_joyNullY)
				{
					plBiped01_ang_force_vec.pan = -plBipedInput01_joyPanConst * ((joy_rot.y-plBipedInput01_joyNullY) / (255-plBipedInput01_joyNullY));
				}
 				if(joy_rot.x < -plBipedInput01_joyNullX || joy_rot.x > plBipedInput01_joyNullX)
				{
					plBiped01_ang_force_vec.tilt = -plBipedInput01_joyTiltConst * ((joy_rot.x-plBipedInput01_joyNullX) / (255-plBipedInput01_joyNullX));
				}



				// raw Z is reduced (to -255) by the right trigger,
				//rasied to 255 by the left

				if(joy_raw.z < -plBipedInput01_joyRawZ_NegTrigger)
				{
 					plBiped01_flag_array[0] |= plBipedInput01_flag0_use_1;	// set flag to use item 1 (i.e. gun)

				}
 				if(joy_raw.z > plBipedInput01_joyRawZ_PosTrigger)
				{
 					plBiped01_flag_array[0] |= plBipedInput01_flag0_use_2;	// set flag to use item 2
 				}

			}
			else
			{

				if(joy_rot.z<-plBipedInput01_joyNullX || joy_rot.z>plBipedInput01_joyNullX)
				{
					// treat gamepad as absolute device which is more
					// like a keyboard than a joystick
					if(plBipedInput01_qJoystick == 1)
					{
					 	plBiped01_ang_force_vec.pan = -sign(joy_rot.z);
					}
					else
					{
						// using analog joystick
						plBiped01_ang_force_vec.pan = -(joy_rot.z-plBipedInput01_joyNullX) / (255-plBipedInput01_joyNullX);
					}
				}

				if(joy_rot.y<-plBipedInput01_joyNullY || joy_rot.y>plBipedInput01_joyNullY)
				{
					// treat gamepad as absolute device which is more
					// like a keyboard than a joystick
					if(plBipedInput01_qJoystick == 1)
					{
					 	plBiped01_ang_force_vec.tilt = -sign(joy_rot.y);
					}
					else
					{
						// using analog joystick
						plBiped01_ang_force_vec.tilt = -(joy_rot.y-plBipedInput01_joyNullY) / (255-plBipedInput01_joyNullY);
					}
				}
			}// end non-XBox style controller

		}


		//--------------------------------------------------------------
		// Non-auto keys (keys that only activate once every few ticks)

		if(plBipedInput01_key_delay <= 0)
		{
/*
			if(Key_Pressed(plBipedInput01_activate_scancode)
				|| Key_Pressed(plBipedInput01_activate_but_scancode))
			{
				plBiped01_flag_array[0] |= plBipedInput01_flag0_activate;	// set flag to preform activation scan
				plBipedInput01_key_delay = plBipedInput01_key_delay_const;
			}
*/

			if(Key_Pressed(plBipedInput01_previtem_but_scancode))
			{
				plBiped01_flag_array[0] |= plBipedInput01_flag0_prev;
				plBipedInput01_key_delay = plBipedInput01_key_delay_const;
			}
			if(Key_Pressed(plBipedInput01_nextitem_but_scancode))
			{
				plBiped01_flag_array[0] |= plBipedInput01_flag0_next;
				plBipedInput01_key_delay = plBipedInput01_key_delay_const;
			}
		}
		else
		{
			plBipedInput01_key_delay -= time_step;
		}

	} // end  on == enable_key



	// mouse_look input
	if((1==plBipedInput01_mouse_look_q) && (ON == enable_mouse))
	{
		// NOTE: the old method don't work due to roundoff errors.
		//    we assumed that the camera view fills the entire screen thus the formula is:
		//				dang = (mickey/screen size)*camera arc
		// The NEW method uses the mouse force value.
//		plBiped01_ang_force_vec.pan -= ((mickey.x/screen_size.x)*camera.arc)*plBipedInput01_mouse_look_factor;
		plBiped01_ang_force_vec.pan -= mouse_force.x*plBipedInput01_mouse_look_factor;

		if(1==plBipedInput01_mouse_look_invt_q)
		{
			plBiped01_ang_force_vec.tilt += mouse_force.y*plBipedInput01_mouse_look_factor;
		}
		else
		{
			plBiped01_ang_force_vec.tilt -= mouse_force.y*plBipedInput01_mouse_look_factor;
		}

	}

	// make sure vectors are within bounds
/*	NOTE: ang_force is a

	if(vec_length(plBiped01_ang_force_vec) > 1)
	{
		vec_normalize(plBiped01_ang_force_vec,1);
	}
*/
	if(vec_length(plBiped01_force_vec) > 1)
	{
		vec_normalize(plBiped01_force_vec,1);
	}



	// force modifiers
	if(Key_Pressed(plBipedInput01_slow_scancode))
	{
		// slow key pressed, reduce walk speed
		vec_scale(plBiped01_force_vec,plBiped01_force_slow_const);
	}





}

endif;